package com.ruoyi.system.controller;

import java.util.List;

import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.system.domain.ActSuppliesHistorydata;
import com.ruoyi.system.service.IActSuppliesHistorydataService;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.common.core.page.TableDataInfo;

/**
 *  历史数据Controller
 * 
 * @author dongjg
 * @date 2023-08-01
 */
@RestController
@RequestMapping("/system/historydata")
public class ActSuppliesHistorydataController extends BaseController
{
    @Autowired
    private IActSuppliesHistorydataService actSuppliesHistorydataService;

    /**
     * 查询 历史数据列表
     */
    @PreAuthorize("@ss.hasPermi('system:historydata:list')")
    @GetMapping("/list")
    public TableDataInfo list(ActSuppliesHistorydata actSuppliesHistorydata)
    {
        startPage();
        List<ActSuppliesHistorydata> list = actSuppliesHistorydataService.selectActSuppliesHistorydataList(actSuppliesHistorydata);
        return getDataTable(list);
    }

    /**
     * 导出 历史数据列表
     */
    @PreAuthorize("@ss.hasPermi('system:historydata:export')")
    @Log(title = " 历史数据", businessType = BusinessType.EXPORT)
    @GetMapping("/export")
    public AjaxResult export(ActSuppliesHistorydata actSuppliesHistorydata)
    {
        List<ActSuppliesHistorydata> list = actSuppliesHistorydataService.selectActSuppliesHistorydataList(actSuppliesHistorydata);
        ExcelUtil<ActSuppliesHistorydata> util = new ExcelUtil<ActSuppliesHistorydata>(ActSuppliesHistorydata.class);
        return util.exportExcel(list, " 历史数据");
    }

    /**
     * 获取 历史数据详细信息
     */
    @PreAuthorize("@ss.hasPermi('system:historydata:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id)
    {
        return AjaxResult.success(actSuppliesHistorydataService.selectActSuppliesHistorydataById(id));
    }

    /**
     * 新增 历史数据
     */
    @PreAuthorize("@ss.hasPermi('system:historydata:add')")
    @Log(title = " 历史数据", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody List<ActSuppliesHistorydata> actSuppliesHistorydata)
    {
        return toAjax(actSuppliesHistorydataService.insertActSuppliesHistorydata(actSuppliesHistorydata));
    }

    /**
     * 修改 历史数据
     */
    @PreAuthorize("@ss.hasPermi('system:historydata:edit')")
    @Log(title = " 历史数据", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody ActSuppliesHistorydata actSuppliesHistorydata)
    {
        return toAjax(actSuppliesHistorydataService.updateActSuppliesHistorydata(actSuppliesHistorydata));
    }

    /**
     * 删除 历史数据
     */
    @PreAuthorize("@ss.hasPermi('system:historydata:remove')")
    @Log(title = " 历史数据", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids)
    {
        return toAjax(actSuppliesHistorydataService.deleteActSuppliesHistorydataByIds(ids));
    }

    /**
     * 添加 历史数据
     */
    @PreAuthorize("@ss.hasPermi('system:historydata:addInsert')")
    @Log(title = " 历史数据", businessType = BusinessType.INSERT)
    @PostMapping("/addInsert")
    public AjaxResult addInsert(@RequestBody List<ActSuppliesHistorydata> actSuppliesHistorydata)
    {
        actSuppliesHistorydataService.addInsertActSuppliesHistorydata(actSuppliesHistorydata);
        return toAjax(1);
    }
}

